@@ -662,7 +662,7 @@ class Balance:
                     account_number, title, 'OUVERTURE %s' % (self._year + 1),
                     disp(deb), disp(cred), 'BILAN %s' % self._year])
         if abs(tot_credit - tot_debit)>decimal.Decimal(epsilon):
-            txt = _('New year opening error ({0}):\n' + \
+            txt = _('New year opening error ({0}):\n'
                     'tot_debit={1} differs from tot_credit={2} !!\n\n')
             sys.stderr.write(txt.format(self._year+1, tot_debit, tot_credit))
             sys.exit(2)
