@@ -91,8 +91,12 @@ def gen_dvi(dico):
     p = subprocess.Popen('latex -version', shell=True, stdout=subprocess.PIPE)
     p.communicate()
     if p.returncode:
-        sys.stderr.write(_('latex command not found, please install' +\
-            ' (MikTeX on Windows, texlive-latex on Linux)\n'))
+        sys.stderr.write(_('latex command not found, please install it.\n'
+            '\tMikTeX on Windows,\n'
+            '\ttexlive-latex on RPM based Linux distributions\n'
+            '\tor texlive-latex-base texlive-lang-french texlive-latex-extra'
+            ' texlive-fonts-recommended'
+            '\t   for Debian based distributions (like Ubuntu)\n'))
         return False
     p = subprocess.Popen('latex -halt-on-error %(base_file)s.tex' % dico,
         cwd=dico['tmp_dir'], shell=True, stdout=subprocess.PIPE)
@@ -103,7 +107,7 @@ def gen_dvi(dico):
     if not rc:
         log_file = dico['root_file'] + '.log'
         if os.path.exists(log_file):
-            for l in open(log_file):
+            for l in open(log_file, 'rb'):
                 print (l)
         sys.exit(2)
     print (_('%(base_file)s.dvi generated.') % dico)
@@ -311,16 +315,17 @@ _nb_bill = %(nb_bill)s
                     [ tjm[1]*tjm[2] for tjm in self._day_price])
         return self._total_excluded_tax
 
+
     def get_VAT(self):
         if not self._total_excluded_tax:
             self._total_excluded_tax = self.get_total_excluded_tax()
         return decimal.Decimal('%.3f' % round(
-                    self._total_excluded_tax * decimal.Decimal(self.get_VAT_rate()), 2))
+            self._total_excluded_tax * decimal.Decimal(self.get_VAT_rate()), 2))
 
     def get_VAT_rate(self):
         if self._year<2014:
-            return 0.196
-        return 0.2
+            return decimal.Decimal('0.196')
+        return decimal.Decimal('0.2')
 
     def get_total_tax_incl(self):
         if not self._total_excluded_tax:
