@@ -24,9 +24,9 @@ import sys
 import datetime
 import shutil
 
-import generate_all
-import config
-from i18n import _
+import facct.generate_all as generate_all
+import facct.config as config
+from facct.i18n import _
 
 def account_dir(orga, year):
     return os.path.join(config.get_accounts_dir(orga), '%s' % year)
