@@ -30,19 +30,15 @@ import time
 import codecs
 
 import argparse
-import config
-from i18n import _
-import bill.gen_tex as gen_tex
+import facct.config as config
+from facct.i18n import _
+import facct.bill.gen_tex as gen_tex
+from facct.IncomeStatement import IncomeStatement
+from facct.IncomeStatement import get_float
 
 gen_tex.setlocale()
 debug=False
 
-def get_float(str):
-    if str:
-        return decimal.Decimal(str.replace(',', '.'))
-    else:
-        return decimal.Decimal(0)
-
 def load_operations(template_file):
     reader = csv.reader(open(template_file, 'r'), delimiter=';', quotechar='"')
     op_numbers = {}
@@ -125,6 +121,7 @@ def get_day(first_date, year, month, date):
 
 
 class Operation:
+    _exception_output = False
     def __init__(self, op_number, first_date, unit_period,
             periodicity, exception):
         self.records = []
@@ -137,10 +134,25 @@ class Operation:
         if not periodicity:
             periodicity = '1'
         if unit_period == 'd':
+            def not_exception(date):
+                return True
+
             def iter_day(date):
                 return (True, date + datetime.timedelta(days=int(periodicity)) )
+            self.not_exception = not_exception
             self.iter_date = iter_day
+
         if unit_period == 'm':
+            def not_exception(D):
+                val = (not exception or D.month not in [
+                        int(e) for e in exception.split()])
+                if not val:
+                    msg_err = 'exception: {0}, D.month={1}, op_number={2},'
+                    msg_err += 'val={3}, D={4}\n'
+                    sys.stdout.write (msg_err.format(
+                                exception, D.month, op_number, val, D))
+                return val
+
             def iter_month(date):
                 month = date.month + int(periodicity)
                 if month < 13:
@@ -149,19 +161,31 @@ class Operation:
                     year = date.year+1
                     month = 1
                 D = get_day(self.first_date, year, month, date)
-                val = (not exception or D.month not in [
+                return D
+
+            self.not_exception = not_exception
+            self.iter_date = iter_month
+
+        if unit_period == 'y':
+            def not_exception(d):
+                val = (not exception or d.year not in [
                         int(e) for e in exception.split()])
                 if not val:
-                    msg_err = 'exception: {0}, D.month={1}, op_number={2},'
-                    msg_err += 'val={3}, D={4}\n'
-                    sys.stdout.write (msg_err.format(
-                                exception, D.month, op_number, val, D))
-                return (val, D)
-            self.iter_date = iter_month
-        if unit_period == 'y':
+                    if config.get_args().vverbose:
+                        msg_err = 'exception: {0}, d.year={1}, op_number={2},'
+                        msg_err += 'val={3}, d={4}\n'
+                        sys.stdout.write (msg_err.format(
+                                exception, d.year, op_number, val, d))
+                    else:
+                        if not Operation._exception_output and config.get_args().verbose:
+                            print ('--vverbose or -w to find out more about journal exceptions.')
+                            Operation._exception_output = True
+                return val
+
             def iter_year(date):
                 d = datetime.date(date.year + int(periodicity), date.month , 1)
-                return (True, d + datetime.timedelta(days=int(date.day)))
+                return d + datetime.timedelta(days=int(date.day))
+            self.not_exception = not_exception
             self.iter_date = iter_year
 
 
@@ -172,11 +196,10 @@ class Operation:
         dates = []
         tmp_date = self.first_date
         limit = datetime.date(tmp_date.year, 12, 31)
-        keep = True
         while tmp_date <= limit:
-            if keep:
+            if self.not_exception(tmp_date):
                 dates.append(tmp_date)
-            (keep, tmp_date) =  self.iter_date(tmp_date)
+            tmp_date =  self.iter_date(tmp_date)
         return dates
 
     def generate(self):
@@ -221,32 +244,24 @@ def concat_dumps(merged_dumps_list, f):
 
     def max_min(a, b):
         return getDate(a[-1][DATE]) < getDate(b[0][DATE])
-    if (j<nb and max_min(f, merged_dumps_list[j])) or\
-            (j==nb and max_min(merged_dumps_list[nb-1], f)):
+
+    if (j==nb and max_min(merged_dumps_list[nb-1], f)):
         return merged_dumps_list.insert(j, f)
 
     if j==nb:
         to_merge = merged_dumps_list.pop(nb-1)
         return concat_dumps(merged_dumps_list, concat_2_segments(to_merge, f))
 
-    to_merge = merged_dumps_list.pop(j)
-    return concat_dumps(merged_dumps_list, concat_2_segments(f, to_merge))
+    if (max_min(f, merged_dumps_list[j])) and \
+        (j==0 or (j>0 and max_min(merged_dumps_list[j-1], f)) ):
+        return merged_dumps_list.insert(j, f)
 
-def merge_dumps(file_dumps):
-    merged_dumps_list = []
-    for f in file_dumps:
-        if debug:
-            print (len(f))
-        if f:
-            concat_dumps(merged_dumps_list, f)
-            if debug:
-                for g in merged_dumps_list:
-                    print ('g=({0}, {1})'.format (
-                                getDate(g[0][DATE]), getDate(g[-1][DATE])))
-    bank_entries = [ ]
-    for f in merged_dumps_list:
-        bank_entries.extend(f)
-    return bank_entries
+    if (max_min(f, merged_dumps_list[j])):
+        to_merge = merged_dumps_list.pop(j-1)
+        return concat_dumps(merged_dumps_list, concat_2_segments(to_merge, f))
+    else:
+        to_merge = merged_dumps_list.pop(j)
+        return concat_dumps(merged_dumps_list, concat_2_segments(f, to_merge))
 
 
 class UnicodeReader:
@@ -381,9 +396,13 @@ class WriteGenerator:
         self._bank_entries = bank_entries
         self._year = year
         self._rows = []
-        self._orphelins = []
+        self._orphans = []
         for e in self._bank_entries:
-            e[LABEL] = ' '.join(e[LABEL].split('\n')[0].split()[1:-1])
+            splitted = e[LABEL].split('\n')[0].split()
+            if splitted and splitted[0] == 'CHEQUE':
+                e[LABEL] = ' '.join(splitted)
+            else:
+                e[LABEL] = ' '.join(splitted[1:-1])
 
     def compute(self):
         essence = ( 'Essence',
@@ -391,10 +410,12 @@ class WriteGenerator:
                 ('401100', 'Fournisseurs - Achats de biens et services'),
                 ('512100', 'Banque'))
         peage = ('Péage', essence[1], essence[2], essence[3])
+        train = ('Train', essence[1], essence[2], essence[3])
         categories = {
             peage: ('COFIROUTE',),
             essence: ('PAREA CONFL DAC', 'PAREA CONFL STA', 'REL. CLAIR BOIS',
                 'STATION BP ROC', 'COFIROUTE', 'E.LECLERC DAC', 'ESSO PAVE'),
+            train: ('SNCF',),
         }
         self._rows = []
         def get_row(cat, entry, transit_account=True, bill_time='facture'):
@@ -422,24 +443,46 @@ class WriteGenerator:
                     break
             self._rows.extend(ops)
             if not ops:
-                self._orphelins.append(e)
-                print (e)
+                self._orphans.append(e)
+        if self._orphans:
+            sys.stderr.write ('These bank entries have not been found'
+                    ' in the list of registered patterns:\n')
+            for e in self._orphans:
+                sys.stderr.write ('%s\n' % str(e))
 
 def get_account_dir(orga):
     return config.get_accounts_dir(orga)
 
 class Journal:
     def __init__(self, orga, year):
-        self.orga = orga
-        self.year = year
+        self._orga = orga
+        self._year = year
         self.entries = []
         self.ops = {}
+        self.bank_lists = []
+
+    def merge_dumps(self, file_dumps):
+        merged_dumps_list = self.bank_lists
+        for f in file_dumps:
+            if debug:
+                print (len(f))
+            if f:
+                concat_dumps(merged_dumps_list, f)
+                if debug:
+                    for g in merged_dumps_list:
+                        print ('g=({0}, {1})'.format (
+                                    getDate(g[0][DATE]), getDate(g[-1][DATE])))
+        bank_entries = [ ]
+        for f in merged_dumps_list:
+            bank_entries.extend(f)
+        return bank_entries
+
 
     def add(self, records):
         self.ops.update(records)
 
     def get_account_dir(self):
-        return os.path.join(get_account_dir(self.orga), str(self.year))
+        return os.path.join(get_account_dir(self._orga), str(self._year))
 
     def sort(self):
         self.entries.sort(key=lambda record: record.date)
@@ -459,13 +502,13 @@ class Journal:
         self.entries = [e for e in self.entries if not e.row[-1] == 'DUPLICATE']
         self.sort()
 
-    def corpTaxWritting(self, tax):
-        r1 = Record(datetime.date(self.year, 12, 31),
+    def corpTaxWriting(self, tax):
+        r1 = Record(datetime.date(self._year, 12, 31),
                 ['695000', 'Impôts sur les bénéfices\nEstimation',
-                'IS_%s' % self.year, '%s' % tax, '', "Ecriture d'inventaire"])
-        r2 = Record(datetime.date(self.year, 12, 31),
+                'IS_%s' % self._year, '%s' % tax, '', "Ecriture d'inventaire"])
+        r2 = Record(datetime.date(self._year, 12, 31),
                 ['448600', 'Etat-Charges à payer',
-                'IS_%s' % self.year, '', '%s' % tax, "Ecriture d'inventaire"])
+                'IS_%s' % self._year, '', '%s' % tax, "Ecriture d'inventaire"])
         self.entries.append(r1)
         self.entries.append(r2)
 
@@ -479,22 +522,14 @@ class Journal:
                 gross_profit += get_float(e.row[CREDIT]) - get_float(e.row[DEBIT])
         return gross_profit
 
-    def corpTaxCalculation(self, gross_profit):
-        tax = 0
-        SS_ceiling = 38120
-        if gross_profit>=0:
-            tax = gross_profit*15/100
-        if gross_profit>=SS_ceiling:
-            tax += (gross_profit-SS_ceiling)/3
-        return decimal.Decimal( '%.2f' % round(tax,2))
-
     def computCorpTax(self):
         gross_profit = self.profitCalculation()
-        tax = self.corpTaxCalculation(gross_profit)
-        self.corpTaxWritting(tax)
+        tax = IncomeStatement(None, self._orga,
+                self._year).corpTaxCalculation(gross_profit)
+        self.corpTaxWriting(tax)
 
     def register(self):
-        file_name = os.path.join(self.get_account_dir(), '%s.csv' % self.year)
+        file_name = os.path.join(self.get_account_dir(), '%s.csv' % self._year)
         writer = csv.writer(open(file_name, 'w'), delimiter=',', quotechar='"')
         header = ['N°', 'Date', 'Comptes', 'Intitulé', 'Code opération',
              'Débit', 'Crédit', 'Réf. Pièce']
@@ -524,7 +559,7 @@ class Journal:
         if getDate(je['Date'])> date_min:
             sys.stderr.write(_(
                 'Please check bank journal entries between {0}-01-01 and {1}\n'
-                    ).format(date_min.year, date_min))
+                    ).format(date_min._year, date_min))
             sys.stderr.write(_('Total should be {0} on {1}\n').format(
                         be[AMOUNT], date_min))
             sys.exit(2)
@@ -557,7 +592,7 @@ class Journal:
             'Please check bank journal entries between {0}-01-01 and {1}\n'
                 ).format(date_min.year, date_min))
         sys.stderr.write(_(
-            'Total should be {0} on {1} and is in journal {2} at this time.\n'
+            'Total should be {0} on {1} and is {2} in journal at this time.\n'
                 ).format (be[AMOUNT], date_min, total))
         sys.exit(2)
 
@@ -566,37 +601,38 @@ class Journal:
             'Débit': e.row[DEBIT], 'Crédit': e.row[CREDIT],
             'Code opération': e.row[CODE_OP]}
             for e in self.entries if e.row[COMPTES] == '512100']
+
         if not bank_entries or not self.bankInJnl:
             return
-        be = bank_entries[0]
-        if not be[CODE] == '07':
-            return
-        je = self.bankInJnl[0]
-        date_min = getDate(be[DATE])
-        b=0
-        j = 0
-        log = []
-        total = 0
-        while b < len(bank_entries):
-            be = bank_entries[b]
-            date_min = getDate(be[DATE])
-            while j<len(self.bankInJnl) and\
-                getDate(self.bankInJnl[j]['Date'])<=date_min:
-                je = self.bankInJnl[j]
-                deb = get_float(je['Débit'])
-                cred = get_float(je['Crédit'])
-                date = je['Date']
-                total += deb - cred
-                if float(be[AMOUNT])-float(total):
-                    log.append([je, be, total])
-                j += 1
-            b += 1
 
-        if not log:
+        if not bank_entries[0][CODE] == '07':
             return
-        sys.stdout.write(_(
-        '[OK] The last same total is {0} in bank ({1}) and in journal ({2}).\n'
+
+        log = []
+        EQUAL, DIFFERENT = range(2)
+        last_status = DIFFERENT
+        for be in bank_entries:
+            date_bank = getDate(be[DATE])
+            jnl_amount = 0
+            for  je in self.bankInJnl:
+                if getDate(je['Date']) > date_bank:
+                    break
+                jnl_amount += get_float(je['Débit']) - get_float(je['Crédit'])
+                last_status = DIFFERENT
+                if float(be[AMOUNT]) == float(jnl_amount):
+                    log.append([je, be, jnl_amount])
+                    last_status = EQUAL
+
+        if last_status == EQUAL:
+            return
+
+        if log:
+            sys.stdout.write(_(
+            '[OK] The last same total is {0} in bank ({1}) and in journal ({2}).\n'
             ).format( log[-1][1][AMOUNT], log[-1][1][DATE], log[-1][0]['Date']))
+        else:
+            sys.stdout.write(
+                    '[KO] Bank and journal balances have never been equal\n')
 
     def compareResult(self, bank_entries, jnl_slide):
         self.bank = [ list(e.values()) for e in bank_entries\
@@ -604,23 +640,58 @@ class Journal:
         self.jnl = [ list(e.values()) for e in jnl_slide\
                    if not e[MARKED] and e[CODE] == '04' and e[AMOUNT]]
 
+    def isInOneInterval(self, jnl_entry):
+        for f in self.bank_lists:
+            min_date = getDate(f[0][DATE])
+            max_date = getDate(f[-1][DATE])
+            jnl_date = getDate(jnl_entry[DATE])
+            if jnl_date >= min_date and jnl_date <= max_date:
+                return True
+        return False
+
+    def jnlEntrieInBankEntriesIntervals(self):
+        inIntervals = []
+        for e in self.jnl:
+            if self.isInOneInterval(e):
+                inIntervals.append(e)
+
+        if len(inIntervals)>0:
+            print ('\nWe do not try to insert data coming from bank\n' +
+                    'because the Journal template is not up to date.\n\n' +
+                    'Please fix the following journal entries:\n')
+            for e in inIntervals:
+                print (e)
+            print ('')
+            return True
+        return False
+
     def generateFromstatements(self):
         if self.jnl:
-            for e in self.jnl:
-                sys.stdout.write ('Jnl:%s\n'% str(e))
-            for e in self.bank:
-                sys.stdout.write ('Bnk:%s\n'% str(e))
-            # We do not generate content if Jnl not clean, so we return
-            return
+            if config.get_args().vverbose:
+                for e in self.jnl:
+                    sys.stdout.write ('Jnl:%s\n'% str(e))
+                for e in self.bank:
+                    sys.stdout.write ('Bnk:%s\n'% str(e))
+            else:
+                if config.get_args().verbose:
+                    print ('All journal entries are not found in entries loaded'
+                       ' directly from your bank statements you can know \n\tmore'
+                        ' by activating verbose mode: --vverbose (very verbose).')
+
+            # We have to check if the journal's entries are in the date's bank
+            # intervals.
+            if self.jnlEntrieInBankEntriesIntervals():
+                # We do not generate content if Jnl not clean, so we return
+                return
         if self.bank:
-            wGen = WriteGenerator(self.year, self.bank)
+            wGen = WriteGenerator(self._year, self.bank)
             wGen.compute()
             self.entries.extend(wGen._rows)
             self.sort()
 
     def checkBank(self):
         pattern = os.path.join(self.get_account_dir(), 'hsbc','*hsbc*.csv')
-        bank_entries = merge_dumps([loadBank(f) for f in glob.glob(pattern)])
+        bank_entries = self.merge_dumps([loadBank(f) for f in glob.glob(pattern)])
         if debug:
             print (len(bank_entries))
         if bank_entries:
